import os

from automon.helpers.sanitation import Sanitation as S


class Neo4jConfig:
    def __init__(self):
        self.user = os.getenv('NEO4J_USER')
        self.password = os.getenv('NEO4J_PASSWORD')
        self.hosts = S.list_from_string(os.getenv('NEO4J_SERVERS'))



import os


class SplunkConfig:
    def __init__(self, host: str = os.getenv('SPLUNK_HOST'),
                 port: int = os.getenv('SPLUNK_PORT'),
                 username: str = os.getenv('SPLUNK_USERNAME'),
                 password: str = os.getenv('SPLUNK_PASSWORD')):

        self.host = host if host else 'splunkcloud.com'
        self.port = port if port else 8090
        self.username = username if username else ''
        self.password = password if password else ''

    def __str__(self):
        return f'{self.username}@{self.host}:{self.port}'



import os

from automon.log.logger import Logging

log = Logging(__name__, Logging.ERROR)


class ShodanConfig:
    token = os.getenv('SHODAN_API')

    def __init__(self):
        self._log = Logging(ShodanConfig.__name__, Logging.ERROR)
        self.token = os.getenv('SHODAN_API')

        if not self.token:
            self._log.error(f'Missing SHODAN_API')


class Shodan:
    """Get any Shodan information"""

    def __init__(self, api_key: str = None):
        self.api_key = api_key if api_key else ShodanConfig.token

    # TODO: add shodan geoip
    def request(self):
        pass



import unittest

from automon.integrations.geoip import Geoip


class GeoipTest(unittest.TestCase):
    def test_Geoip(self):
        self.assertTrue(Geoip('10.0.0.1'))
        self.assertIsNone(Geoip('10.0.0.1').geoiptool())

# if __name__ == '__main__':
#     unittest.main()



import os

from automon.log.logger import Logging

log = Logging(__name__)


class ConfigSlack:
    slack_webhook = os.getenv('SLACK_WEBHOOK')
    slack_proxy = os.getenv('SLACK_PROXY')
    slack_token = os.getenv('SLACK_TOKEN')

    if not slack_token:
        log.debug(f'SLACK_TOKEN not set')

    def __init__(self, slack_name: str = None):
        self.slack_name = slack_name if slack_name else ''
        self.slack_webhook = os.getenv('SLACK_WEBHOOK')
        self.slack_proxy = os.getenv('SLACK_PROXY')
        self.slack_token = os.getenv('SLACK_TOKEN')



import datetime


class Cluster(dict):
    def __init__(self, raw_metrics: dict):
        self._all_metrics = raw_metrics
        self._nodes = {
            'total': self._all_metrics.get('_nodes').get('total'),
            'successful': self._all_metrics.get('_nodes').get('successful'),
            'failed': self._all_metrics.get('_nodes').get('failed')
        }

        self.cluster_name = self._all_metrics.get('cluster_name')
        self.nodes = []
        self.metrics = []

        for node in self._all_metrics.get('nodes').items():
            self.nodes.append(node[1])

        for metric in self.nodes:
            self.metrics.append(Metric(metric))


class Metric:
    """

    Example JVM Metric for one node:

    "jvm" : {
        "timestamp" : 1571330469200,
        "uptime_in_millis" : 12251148809,
        "mem" : {
          "heap_used_in_bytes" : 27551308288,
          "heap_used_percent" : 82,
          "heap_committed_in_bytes" : 33216266240,
          "heap_max_in_bytes" : 33216266240,
          "non_heap_used_in_bytes" : 162665664,
          "non_heap_committed_in_bytes" : 204083200,
          "pools" : {
            "young" : {
              "used_in_bytes" : 130099216,
              "max_in_bytes" : 558432256,
              "peak_used_in_bytes" : 558432256,
              "peak_max_in_bytes" : 558432256
            },
            "survivor" : {
              "used_in_bytes" : 1132144,
              "max_in_bytes" : 69730304,
              "peak_used_in_bytes" : 69730304,
              "peak_max_in_bytes" : 69730304
            },
            "old" : {
              "used_in_bytes" : 27420076928,
              "max_in_bytes" : 32588103680,
              "peak_used_in_bytes" : 30895029472,
              "peak_max_in_bytes" : 32588103680
            }
          }
        },
        "threading" : {
          "count" : 141,
          "peak_count" : 223
        },
        "gc" : {
          "collectors" : {
            "young" : {
              "collection_count" : 533686,
              "collection_time_in_millis" : 37099480
            },
            "old" : {
              "collection_count" : 75872,
              "collection_time_in_millis" : 9588732
            }
          }
        },
        "buffer_pools" : {
          "mapped" : {
            "count" : 7988,
            "used_in_bytes" : 3715149748692,
            "total_capacity_in_bytes" : 3715149748692
          },
          "direct" : {
            "count" : 10146,
            "used_in_bytes" : 166764364,
            "total_capacity_in_bytes" : 166764363
          }
        },
        "classes" : {
          "current_loaded_count" : 17808,
          "total_loaded_count" : 18183,
          "total_unloaded_count" : 375
        }
      },


    """

    def __init__(self, node: Cluster):
        self._node = node
        # self.node_name = self._node['name']
        self.node_name = self._node.cluster_name

        self.jvm = self._node.get('jvm')
        self.heap_used_percent = self.jvm.get('mem').get('heap_used_percent')

        self._mem = self.jvm.get('mem')
        self._timestamp = self.jvm.get('timestamp')
        self._uptime_in_millis = self.jvm.get('uptime_in_millis')
        self._gc = self.jvm.get('gc')
        self._threads = self.jvm.get('threading')
        self._buffer_pools = self.jvm.get('buffer_pools')
        self._classes = self.jvm.get('classes')

    def __eq__(self, other):
        if not isinstance(other, Metric):
            return NotImplemented

        return self.jvm == other.jvm


class MetricTimestamp(Metric):
    """Metric checking for Elastcsearch nodes"""

    def __init__(self, metric: Metric):
        self.metric = metric if isinstance(metric, Metric) else Metric(metric)
        self.last_checked = datetime.datetime.now()
        self.node_name = self.metric.node_name
        self.heap_used_percent = self.metric.heap_used_percent

    def _time_now(self):
        return datetime.datetime.now()

    def _is_instance(self, new_metric):

        if not isinstance(new_metric, MetricTimestamp):
            new_metric = MetricTimestamp(new_metric)

        return new_metric

    def change_percent(self, new_metric):
        current = self._is_instance(new_metric).heap_used_percent * .01
        previous = self.heap_used_percent * .01

        return ((float(current) - previous) / previous) * 100

    # def slope(self, new_metric):
    #     new_metric = self._is_instance(new_metric)
    #
    #     y1 = self.metric.heap_used_percent
    #     y2 = new_metric.heap_used_percent
    #
    #     x1 = self.last_checked.timestamp()
    #     x2 = new_metric.last_checked.timestamp()
    #
    #     slope = (y2 - y1) / 1
    #     # time is currently not taken into account
    #     # time is disabled
    #     # slope = (y2 - y1) / (x2 - x1)
    #
    #     return slope

    def __eq__(self, other):
        if not isinstance(other, MetricTimestamp):
            return NotImplemented

        return self.node_name == other.node_name



from flask import Flask

from automon.log import Logging
from automon.integrations.flask.config import ConfigFlask


class FlaskBoilerplate:

    def __init__(self, flask_name=__name__):
        self._log = Logging(FlaskBoilerplate.__name__, Logging.DEBUG)

        self.app = Flask(flask_name)
        self.app = ConfigFlask.javascript_compatibility(self.app)



import os
import slack
import random
import asyncio

from slack.web.slack_response import SlackResponse
from slack.errors import SlackApiError

from automon.log.logger import Logging
from automon.helpers.asyncio_ import AsyncStarter
from automon.integrations.slack.config import ConfigSlack

# TODO: maybe separate class for SlackAuth

log = Logging(__name__, level=Logging.ERROR)


class SlackError:
    def __init__(self, error: SlackApiError):
        """The request to the Slack API failed.
        The server responded with:
        {
            "ok": false,
            "error": "missing_scope",
            "needed": "users:read",
            "provided": "chat:write,chat:write.customize,chat:write.public,links:write,links:read,files:read,files:write"
        }
        """

        self._error = error
        self._response = self._error.response

        self.api_url = self._response.api_url

        self.data = dict(self._response.data)
        self.ok = self.data.get('ok')
        self.error = self.data.get('error')
        self.needed = self.data.get('needed')
        self.provided = self.data.get('provided')

        self.headers = self._response.headers
        self.http_verb = self._response.http_verb
        self.req_args = self._response.req_args
        self.status_code = self._response.status_code

    def __repr__(self):
        return f'{self.data}'

    def __str__(self):
        return self.__repr__()


class BotInfo(SlackResponse):
    def __init__(self, response: dict):
        """{
            "ok": true,
            "bot": {
                "id": "B061F7JD2",
                "deleted": false,
                "name": "beforebot",
                "updated": 1449272004,
                "app_id": "A161CLERW",
                "user_id": "U012ABCDEF",
                "icons": {
                    "image_36": "https://...",
                    "image_48": "https://...",
                    "image_72": "https://..."
                }
            }
        }
        """

        self.status = response.get('ok')
        self.bot = dict(response.get('bot'))
        self.id = self.bot.get('id')
        self.deleted = self.bot.get('deleted')
        self.name = self.bot.get('name')
        self.updated = self.bot.get('updated')
        self.app_id = self.bot.get('app_id')
        self.user_id = self.bot.get('user_id')
        self.icons = self.bot.get('icons')

    def __repr__(self):
        return f'{self.__dict__}'

    def __str__(self):
        return f'{self.__dict__}'

    def __eq__(self, other):
        if not isinstance(other, BotInfo):
            return NotImplemented

        return self.__dict__ == other.__dict__


class Slack(ConfigSlack):
    """
    All Slack interactions
    """

    def __init__(self, token: ConfigSlack.slack_token = None, username: str = None,
                 channel: str = None, icon_emoji: str = None, icon_url: str = None):

        self._log = Logging(Slack.__name__, Logging.DEBUG)

        self.token = token if token else ConfigSlack().slack_token

        if not self.token:
            self._log.error(f'Missing SLACK_TOKEN')
            self.client = None
        else:
            self.client = slack.WebClient(token=self.token)

        # TODO: use token to get bot info
        self.username = username if username else self._get_bot_info()
        # automonbot-test
        self.channel = channel if channel else ''
        self.icon_emoji = icon_emoji if icon_emoji else ''
        self.icon_url = icon_url if icon_url else ''

        # start consumer
        self._start_loop = AsyncStarter()
        self.queue = self._start_loop.queue
        asyncio.create_task(self._consumer())
        self._stop = False

        # TODO: integrate slacklog
        # self.slacklog = SlackLogging(token)

    def _get_bot_info(self: SlackResponse):
        if self.client:
            try:
                name = BotInfo(self.client.bots_info()).name
                self._log.debug(f'Bot name: {name}')
                return name
            except Exception as e:
                error = SlackError(e)
                self._log.error(
                    f'''{self._get_bot_info.__name__}\tCouldn't get bot name, missing permission: {error.needed}''',
                    enable_traceback=False)
                return ''
        return ''

    def _run_until_complete(self):
        while True:
            if self.queue.qsize():
                asyncio.run(asyncio.sleep(0))
            else:
                break
        self._stop = True

    async def chat_postMessage(self, channel: str, text: str) -> slack.WebClient.chat_postMessage:

        if not self.client:
            return False

        if not text:
            return SyntaxError

        await self.queue.put((channel, text))

    def files_upload(self, file, filename=None):
        """
        Upload file to slack

        note: username does nothing
        note: icon_emoji does nothing
        note: title takes precedent over filename
        note: title will display full string
        note: filename displays only filename, no extension

        :param file: path to file
        :param filename: (Optional) filename
        :return: Slack response or False
        """

        if not self.client:
            return False

        # check if file exists
        if not os.path.isfile(file):
            self._log.error(f'File not found: {file}')
            self._log.error(f'Working dir: {os.getcwd()}')
            return False

        # get filename
        if not filename:
            filename = os.path.split(file)

        # get file size
        file_size = os.path.getsize(file)

        title = f'{filename} ({file_size}B'

        # TODO: add string validation for `filename`
        # TODO: find a way to change username for files_upload

        response = self.client.files_upload(
            file=file, filename=filename, title=title, username=self.username, channels=self.channel)

        assert response["ok"]

        self._log.debug(f'File uploaded: {file} ({file_size}B) ({self.username}')

        return response

    async def _consumer(self):

        burst = 0
        burst_max = 0
        retry = 0
        while True:
            channel, text = await self.queue.get()

            if self._stop:
                break

            msg = f'{channel} @{self.username}: {text}'

            while True:
                try:
                    self._log.debug(msg)
                    response = self.client.chat_postMessage(
                        text=text, channel=channel, username=self.username,
                        icon_emoji=self.icon_emoji, icon_url=self.icon_url)
                    assert response["ok"]

                    if not retry and self.queue.qsize() > 10:
                        # if a lot in the queue, try bursting
                        await asyncio.sleep(random.choice(range(2)))
                    else:
                        sleep = random.choice(range(4))
                        self._log.debug(f'sleeping {sleep}, queue size is {self.queue.qsize()}')
                        await asyncio.sleep(sleep)

                    self._log.debug(f'Burst: {burst}, Retry: {retry}, Queue {self.queue.qsize()}')

                    burst += 1
                    retry = 0

                    break
                except Exception as e:
                    if retry > 5:
                        break

                    retry += 1
                    burst_max = burst
                    error = SlackError(e)
                    self._log.error(
                        f'{self._consumer.__name__}\t{error.error}\t{msg}\tRetry: {retry}, Burst max: {burst_max}',
                        enable_traceback=False)
                    burst = 0



import splunklib.client as client

from automon.integrations.splunk.config import SplunkConfig


class SplunkClient:
    def __init__(self):
        self.config = SplunkConfig()
        self.client = client.connect(
            host=self.config.host,
            port=self.config.port,
            username=self.config.username,
            password=self.config.password)

        # referred to as a service in docs
        self.service = self.client
        assert isinstance(self.service, client.Service)

        self.apps = self.client.apps

    def get_client(self):
        return self.client

    def get_service(self):
        return self.client.connect()

    def get_apps(self):
        return [x for x in self.service.apps]


    def create_app(self, app_name):
        return self.apps.create(app_name)

    def get_app(self, app_name):
        return self.apps[app_name]

    def delete_app(self, app_name):
        return self.apps.delete(app_name)

    def app_info(self, app_name):
        return f'{self.apps[app_name]}'


    def __str__(self):
        return f'{self.client}'



import requests
import elasticsearch

from automon.log.logger import Logging
from automon.integrations.elasticsearch.config import ElasticsearchConfig


class ElasticsearchClient(ElasticsearchConfig):
    def __init__(self, config: ElasticsearchConfig = None):
        self._log = Logging(ElasticsearchClient.__name__, Logging.DEBUG)

        self.config = config if isinstance(config, ElasticsearchConfig) else ElasticsearchConfig()
        self.client = self.config.Elasticsearch
        self.connected = self.client.ping() if self.client else False

        self.indices = []

    def rest(self, url: str) -> requests:
        try:
            return requests.get(url).content
        except Exception as e:
            self._log.error(f'REST request failed: {e}')
            return False

    def ping(self):
        try:
            return self.client.ping()
        except Exception as e:
            self._log.error(f'Ping failed: {e}')
            return False

    def delete_index(self, index):
        try:
            return self.client.delete(index)
        except Exception as e:
            self._log.error(f'Delete index failed: {e}')
            return False

    # def delete_indices(self, index_pattern):
    #     """Requires user interaction"""
    #
    #     if not self.connected:
    #         return False
    #
    #     retrieved_indices = [x for x in self.client.search_indices(index_pattern).keys()]
    #     num_indices = len(retrieved_indices)
    #
    #     if not num_indices:
    #         self._log.debug(f'No indices found')
    #         return False
    #
    #     self._log.info(f'Search found {num_indices} indices')
    #
    #     for index in retrieved_indices:
    #         self._log.debug(index)
    #
    #     # TODO: Find a way to undo index deletions
    #     #       One way could be to rename the indices and store a link to the new
    #     #       indices in a node of deleted indices
    #     if num_indices < 2:
    #         msg = f"\nYOU'RE ABOUT TO DELETE {num_indices} INDEX! ARE YOU SURE YOU WANT TO CONTINUE? "
    #     elif num_indices > 1:
    #         msg = f"\nYOU'RE ABOUT TO DELETE {num_indices} INDICES! ARE YOU SURE YOU WANT TO CONTINUE? "
    #     msg += 'THIS CANNOT BE UNDONE! DECIDED WISELY [y/N]'
    #     print(msg)
    #
    #     answer = str(input()).lower()
    #
    #     if not answer:
    #         answer = 'N'
    #
    #     if answer == 'y':
    #         for index in retrieved_indices:
    #             msg = f'Deleting {index}...'
    #             print(msg, end='')
    #             # Delete the index
    #             self.delete_index(index)
    #             print('done')
    #     else:
    #         print('Whew, you might have just blew it, if you had said yes')

    def search_indices(self, index_pattern):
        try:
            retrieved_indices = self.client.indices.get(index_pattern)
            num_indices = len(retrieved_indices)
            self._log.info(f'Search found {num_indices} indices')
            return retrieved_indices
        except elasticsearch.exceptions.NotFoundError:
            self._log.error(
                f"You provided the index pattern '{index_pattern}', but returned no results")
        except Exception as e:
            self._log.error(f'Failed to search indices: {e}')

    def get_indices(self):
        try:
            retrieved_indices = self.client.indices.get('*')
            self.indices.extend(retrieved_indices)
            self._log.info(f'Retrieved {len(retrieved_indices)} indices')
        except Exception as e:
            self._log.error(f'Failed to get indices: {e}')



import os

from elasticsearch import Elasticsearch, RequestsHttpConnection

from automon.log.logger import Logging
from automon.helpers.sanitation import Sanitation as S


class ElasticsearchConfig:
    def __init__(self, endpoints: str = None, proxy=None,
                 request_timeout: int = 1,
                 http_auth=None,
                 use_ssl: bool = True,
                 verify_certs: bool = True,
                 connection_class: RequestsHttpConnection = RequestsHttpConnection):
        self._log = Logging(ElasticsearchConfig.__name__, Logging.DEBUG)

        self.es_hosts = S.list_from_string(endpoints) if endpoints else \
            S.list_from_string(os.getenv('ELASTICSEARCH_ENDPOINT')) or \
            S.list_from_string(os.getenv('ELASTICSEARCH_HOSTS')) or None

        self.es_proxy = proxy
        self.request_timeout = request_timeout
        self.http_auth = http_auth
        self.use_ssl = use_ssl
        self.verify_certs = verify_certs
        self.connection_class = connection_class

        if self.es_hosts:
            self.Elasticsearch = Elasticsearch(
                hosts=self.es_hosts,
                request_timeout=self.request_timeout,
                http_auth=self.http_auth,
                use_ssl=self.use_ssl,
                verify_certs=self.verify_certs,
                connection_class=self.connection_class)

        else:
            self.Elasticsearch = False
            self._log.error(f'Missing ELASTICSEARCH_ENDPOINT, ELASTICSEARCH_ENDPOINT')

    def __eq__(self, other):
        if not isinstance(other, ElasticsearchConfig):
            self._log.error(f'Not implemented')
            return NotImplemented

        return self.es_hosts == other.es_hosts


class SnapshotBot:
    def __init__(self):
        self.slack_name = 'Elasticsearch Daily Monitor bot'
        self.es_hosts = ElasticsearchConfig().es_hosts
        self.es_repository = os.getenv('ELASTICSEARCH_REPOSITORY')
        self.snapshots_prefix = os.getenv('SNAPSHOTS_PREFIX')


class JVMBot:
    def __init__(self):
        self.slack_name = 'Elasticsearch JVM Monitor bot'
        self.es_hosts = ElasticsearchConfig().es_hosts



import unittest

from automon.integrations.elasticsearch.jvm import ElasticsearchJvmMonitor
from automon.integrations.elasticsearch.config import ElasticsearchConfig, SnapshotBot, JVMBot
from automon.integrations.elasticsearch.client import ElasticsearchClient
from automon.integrations.elasticsearch.cleanup import Cleanup
from automon.integrations.elasticsearch.metrics import Metric, MetricTimestamp, Cluster
from automon.integrations.elasticsearch.snapshots import Snapshot, SnapshotError, ElasticsearchSnapshotMonitor


class ElasticsearchTest(unittest.TestCase):

    def test_ElasticsearchJvmMonitor(self):
        self.assertTrue(ElasticsearchJvmMonitor)
        self.assertTrue(ElasticsearchJvmMonitor())
        self.assertFalse(ElasticsearchJvmMonitor().get_metrics())
        # self.assertFalse(ElasticsearchJvmMonitor().read_file())

    def test_Snapshot(self):
        self.assertTrue(Snapshot)
        self.assertTrue(Snapshot({}))
        self.assertEqual(Snapshot({}), Snapshot({}))
        self.assertNotEqual(Snapshot({}), None)

    def test_SnapshotError(self):
        error = {
            'error': {
                'test': 'test',
                'caused_by': {
                    'type': None,
                    'reason': None,
                    'caused_by': {
                        'type': None,
                        'reason': None
                    }
                }
            }
        }

        self.assertTrue(SnapshotError)
        self.assertTrue(SnapshotError({}))
        self.assertEqual(SnapshotError({}), SnapshotError({}))
        self.assertNotEqual(SnapshotError({}), None)
        self.assertTrue(SnapshotError(error))

    def test_ElasticsearchSnapshotMonitor(self):
        e = ElasticsearchSnapshotMonitor('test', 'test', 'test')

        self.assertTrue(ElasticsearchSnapshotMonitor)
        self.assertTrue(e)
        self.assertFalse(e.check_snapshots())

    def test_ElasticsearchClient(self):
        config = ElasticsearchConfig(None)

        e = ElasticsearchClient(config)

        self.assertTrue(ElasticsearchClient)
        self.assertTrue(e)
        self.assertFalse(e.ping())
        self.assertFalse(e.delete_index(None))
        # self.assertFalse(e.delete_indices(None))
        self.assertFalse(e.search_indices(None))
        self.assertFalse(e.get_indices())

    def test_Cleanup(self):
        self.assertFalse(Cleanup().get_indices())
        self.assertFalse(Cleanup().search_indices(f''))
        # self.assertFalse(Cleanup().delete_indices(f''))

    def test_ElasticsearchConfig(self):
        self.assertTrue(ElasticsearchConfig())
        self.assertEqual(ElasticsearchConfig(), ElasticsearchConfig())

    def test_SnapshotBot(self):
        self.assertTrue(SnapshotBot())

    def test_JVMBot(self):
        self.assertTrue(JVMBot())

    def test_Cluster(self):
        self.assertTrue(Cluster)

    def test_MetricTimestamp(self):
        # metric = Metric()
        self.assertTrue(MetricTimestamp)
        # self.assertTrue(MetricTimestamp(metric)._time_now())

    def test_Metric(self):
        node = None
        jvm = {
            "timestamp": 1571330469200,
            "uptime_in_millis": 12251148809,
            "mem": {
                "heap_used_in_bytes": 27551308288,
                "heap_used_percent": 82,
                "heap_committed_in_bytes": 33216266240,
                "heap_max_in_bytes": 33216266240,
                "non_heap_used_in_bytes": 162665664,
                "non_heap_committed_in_bytes": 204083200,
                "pools": {
                    "young": {
                        "used_in_bytes": 130099216,
                        "max_in_bytes": 558432256,
                        "peak_used_in_bytes": 558432256,
                        "peak_max_in_bytes": 558432256
                    },
                    "survivor": {
                        "used_in_bytes": 1132144,
                        "max_in_bytes": 69730304,
                        "peak_used_in_bytes": 69730304,
                        "peak_max_in_bytes": 69730304
                    },
                    "old": {
                        "used_in_bytes": 27420076928,
                        "max_in_bytes": 32588103680,
                        "peak_used_in_bytes": 30895029472,
                        "peak_max_in_bytes": 32588103680
                    }
                }
            },
            "threading": {
                "count": 141,
                "peak_count": 223
            },
            "gc": {
                "collectors": {
                    "young": {
                        "collection_count": 533686,
                        "collection_time_in_millis": 37099480
                    },
                    "old": {
                        "collection_count": 75872,
                        "collection_time_in_millis": 9588732
                    }
                }
            },
            "buffer_pools": {
                "mapped": {
                    "count": 7988,
                    "used_in_bytes": 3715149748692,
                    "total_capacity_in_bytes": 3715149748692
                },
                "direct": {
                    "count": 10146,
                    "used_in_bytes": 166764364,
                    "total_capacity_in_bytes": 166764363
                }
            },
            "classes": {
                "current_loaded_count": 17808,
                "total_loaded_count": 18183,
                "total_unloaded_count": 375
            }
        }
        self.assertTrue(Metric)
        self.assertRaises(TypeError, Metric)

# if __name__ == '__main__':
#     unittest.main()



from string import (ascii_letters, digits)


class Sanitation:

    @staticmethod
    def hyphen_underscore(text: str) -> str:
        allowed_chars = [x for x in ''.join([ascii_letters, digits, '-', '_'])]

        text = f'{text}'

        return ''.join([x for x in text if x in allowed_chars])

    @staticmethod
    def hyphens_only(text: str) -> str:
        allowed_chars = [x for x in ''.join([ascii_letters, digits, '-'])]

        text = f'{text}'

        return ''.join([x for x in text if x in allowed_chars])

    @staticmethod
    def underscores_only(text: str) -> str:
        allowed_chars = [x for x in ''.join([ascii_letters, digits, '_'])]

        text = f'{text}'

        return ''.join([x for x in text if x in allowed_chars])

    @staticmethod
    def strip_quotes(text: str) -> str:
        removed_chars = ['"', "'"]

        text = str(text)

        for char in removed_chars:
            text = text.strip(char)

        return text

    @staticmethod
    def strip_spaces(text: str) -> str:
        removed_chars = ' '

        text = str(text)
        text = text.strip(removed_chars)

        return text

    @staticmethod
    def strip(text: str) -> str:
        return str(text).strip()

    @staticmethod
    def strip_spaces_from_list(lst: list) -> list:
        removed_chars = ' '

        new_list = list()

        for item in lst:
            item = str(item)
            item = item.strip(removed_chars)
            new_list.append(item)

        return new_list

    @staticmethod
    def safe_string(text: str) -> str:
        allowed_characters = ascii_letters + digits + '-_.'

        text = str(text).strip()
        new_text = []

        for character in text:
            if character in allowed_characters:
                new_text.append(character)
            else:
                new_text.append('_')

        return ''.join(new_text)

    @staticmethod
    def safe_string_allow_spaces(text: str) -> str:
        allowed_characters = ascii_letters + digits + '-_. '

        text = str(text).strip()
        new_text = []

        for character in text:
            if character in allowed_characters:
                new_text.append(character)
            else:
                new_text.append('_')

        return ''.join(new_text)

    @staticmethod
    def safe_filename(text: str) -> str:
        allowed_characters = ascii_letters + digits + '''-_.:?'" '''

        text = str(text).strip()
        new_text = []

        for character in text:
            if character in allowed_characters:
                new_text.append(character)
            else:
                new_text.append('_')

        return ''.join(new_text)

    @staticmethod
    def safe_foldername(text: str) -> str:
        allowed_characters = ascii_letters + digits + '''-_. '''

        text = str(text).strip()
        new_text = []

        for character in text:
            if character in allowed_characters:
                new_text.append(character)
            else:
                new_text.append('_')

        return ''.join(new_text)

    @staticmethod
    def dedup(lst: list) -> list:
        """deduplicate list"""

        new_list = []
        for item in lst:
            if item not in new_list:
                new_list.append(item)

        return new_list

    @staticmethod
    def list_from_string(string: str) -> list:
        if not string:
            return list()

        string = Sanitation.strip_spaces(string)

        new_lst = list()

        if ',' in string:
            new_lst = string.split(',')
        elif ' ' in string:
            new_lst = string.split(' ')
        else:
            string = Sanitation.strip_quotes(string)
            string = Sanitation.strip_spaces(string)
            new_lst.append(string)
            return new_lst

        new_lst = [Sanitation.strip_quotes(item) for item in new_lst]
        new_lst = [Sanitation.strip_spaces(item) for item in new_lst]

        return Sanitation.strip_spaces_from_list(new_lst)



import neo4j

from urllib.parse import urlencode
from datetime import datetime, timezone
from neo4j import GraphDatabase

from automon.log.logger import Logging
from automon.integrations.neo4j.config import Neo4jConfig
from automon.helpers.assertions import assert_label

log = Logging(__name__, Logging.DEBUG)


class Neo4jWrapper:
    """Neo4j wrapper"""

    def __init__(self, config: Neo4jConfig = None) -> neo4j:

        self.config = config if isinstance(config, Neo4jConfig) else Neo4jConfig()

        self.user = self.config.user
        self.password = self.config.password
        self.hosts = self.config.hosts

        for server in self.hosts:
            try:
                self.neo4j = GraphDatabase.driver(server, auth=(self.user, self.password))
                self.driver = self.neo4j
                log.info(f'Connected to neo4j server: {server}')
            except:
                self.neo4j = None
                log.error(f'Cannot connect to neo4j server: {server}')
        else:
            self.neo4j = None

    def _http_header(self, headers) -> dict:
        # [print(x) for x in auth.request_headers(request)]

        # token = helper_brain.hash_key(sorted([x for x in headers]))

        args = dict(
            blob=sorted(headers),
            label='Headers'
        )

        return self._prepare_dict(**args)

    def _prepare_dict(self, blob: dict) -> dict:
        """All inputs first needs to dicts"""
        try:
            return dict(blob)
        except:
            return dict(raw=urlencode(blob))

    def _consolidate(self, query: list) -> list:
        """Join cypher queries list into a string"""
        return '\n'.join(query).strip()

    def _send(self, cypher: str) -> GraphDatabase.driver:
        """This is the query that will be run on the database. So make sure by the time it
        gets to this function all prior checks have passed. Also, create a last check in
        this function for general cypher query-ness"""

        if self.neo4j is None:
            return

        with self.neo4j.session() as session:
            results = session.run(cypher)

        log.debug(f'Cypher: {cypher}')
        log.debug(f'Results: {results}')

        # TODO: print records from cypher response
        # for record in results:
        #     for value in record._values:
        #         for props in value.properties:
        #             print(props, value.properties[props])

        return results

    def send_data(self, label, data):
        """Just take the entry and put it into the database to be parsed later"""

        timestamp_date = datetime.now(tz=timezone.utc).isoformat()
        label = assert_label(label)

        node = 'NODE'
        dict_blob = self._prepare_dict(data)

        # cypher_assertions = []
        # TODO: find a way to check if assertion has already been made
        # query = 'CREATE CONSTRAINT ON ( {node} {label} ) ASSERT {node}.raw IS UNIQUE'
        # query = query.format(node=node, label=label)
        # cypher_assertions.append(query)  # Pre cypher query

        query = list()
        query.append(f'MERGE ( {node} {label} ')
        query.append('{')

        # iterate dict keys
        i = 0
        for key in dict_blob.keys():
            i = i + 1
            value = dict_blob[key]

            if i < len(dict_blob.keys()):
                query.append(f'`{key}`: {value},')
            else:
                query.append(f'`{key}`: {value}')

        query.append(' } )')

        # timestamp of first seen
        # query.append('ON CREATE SET {}.timestamp = "{}"'.format(node, timestamp_date))

        # timestamp of every time seen
        query.append('SET {}.timestamp = "{}"'.format(node, timestamp_date))
        query.append('RETURN *')

        final_cypher = self._consolidate(query)  # self._consolidate sets of queries into one single related query

        log.debug(final_cypher)

        return self._send(final_cypher)

    def create_relationship(self, cypher):
        """Create relationship"""
        log.debug(cypher)
        return self._send(cypher)



import json
import requests

from automon.log import Logging
from automon.integrations.elasticsearch.client import ElasticsearchClient
from automon.integrations.elasticsearch.config import ElasticsearchConfig


class Snapshot:
    def __init__(self, snapshot: dict):
        self._log = Logging(Snapshot.__name__, Logging.DEBUG)

        self.snapshot = snapshot
        self.id = snapshot.get('id')
        self.status = snapshot.get('status')
        self.start_epoch = snapshot.get('start_epoch')
        self.start_time = snapshot.get('start_time')
        self.end_epoch = snapshot.get('end_epoch')
        self.end_time = snapshot.get('end_time')
        self.duration = snapshot.get('duration')
        self.indices = snapshot.get('indices')
        self.successful_shards = snapshot.get('successful_shards')
        self.failed_shards = snapshot.get('failed_shards')
        self.total_shards = snapshot.get('total_shards')

    def __eq__(self, other):
        if not isinstance(other, Snapshot):
            self._log.error(NotImplemented)
            return NotImplemented

        return self.snapshot == other.snapshot


class ElasticsearchSnapshotMonitor:
    def __init__(self, endpoint: str, elasticsearch_repository: str, snapshots_prefix: str):
        self._log = Logging(ElasticsearchSnapshotMonitor.__name__, Logging.DEBUG)

        self._config = ElasticsearchConfig(endpoint)
        self._client = ElasticsearchClient(self._config)

        self._endpoint = self._client.config.es_hosts
        self._repository = elasticsearch_repository
        self._snapshots_prefix = snapshots_prefix

        self.snapshots = []
        self.total_snapshots = None
        self.good_snapshots = []
        self.bad_snapshots = []
        self.error = None

    def _get_all_snapshots(self) -> bool:
        url = f'{self._endpoint}/_cat/snapshots/{self._repository}?format=json&pretty'

        self._log.info('Downloading snapshots list')
        content = self._client.rest(url)

        if content:
            snapshots = json.loads(content)
            return self._process_snapshots(snapshots)

        return False

    def _process_snapshots(self, snapshots: dict) -> bool:
        self._log.info('Processing snapshots')

        try:
            self.total_snapshots = list(snapshots).__len__()

            self._log.info(f'{self.total_snapshots} snapshots')

            for snapshot in snapshots:

                s = Snapshot(snapshot)

                id = s.id
                status = s.status

                if self._snapshots_prefix in id:

                    self.snapshots.append(s)

                    if status == 'SUCCESS' or status == 'success':
                        self.good_snapshots.append(s)
                    else:
                        self.bad_snapshots.append(s)

            return True

        except:
            self._log.error('Unable to get snapshots')
            self.error = SnapshotError(snapshots)
            return False

    def read_file(self, file_path):
        self._log.info('Reading snapshots from file')

        with open(file_path, 'rb') as snapshots:
            snapshots = json.load(snapshots)

        self._process_snapshots(snapshots)

    def check_snapshots(self):
        self._log.info('Checking snapshots')
        return self._get_all_snapshots()


# class ElasticsearchSnapshotMonitorDepreciated:
#     def __init__(self, elasticsearch_endpoint, elasticsearch_repository, snapshots_prefix):
#
#         self.endpoint = elasticsearch_endpoint
#         self.repository = elasticsearch_repository
#         self.snapshots_prefix = snapshots_prefix
#         self.snapshots = []
#         self.good_snapshots = []
#         self.bad_snapshots = []
#
#     def _get_all_snapshots(self, file=None):
#         url = f'{self.endpoint}/_cat/snapshots/{self.repository}?format=json&pretty'
#
#         if file:
#             with open(file, 'rb') as snapshots:
#                 snapshots = json.load(snapshots)
#         else:
#             snapshots = requests.get(url).content
#             snapshots = json.loads(snapshots)
#
#         for snapshot in snapshots:
#
#             s = Snapshot(snapshot)
#             id = s.id
#             status = s.status
#
#             if self.snapshots_prefix in id:
#
#                 self.snapshots.append(s)
#
#                 if status == 'SUCCESS' or status == 'success':
#                     self.good_snapshots.append(s)
#                 else:
#                     self.bad_snapshots.append(s)
#
#     def read_file(self, file):
#         self._get_all_snapshots(file)
#
#     def check_snapshots(self):
#         self._get_all_snapshots()


class SnapshotError:
    def __init__(self, error: dict):
        self._log = Logging(SnapshotError.__name__, Logging.DEBUG)

        self.error = error.get('error')

        if self.error:
            self.root_cause = self.error.get('root_cause')
            self.type = self.error.get('type')
            self.reason = self.error.get('reason')
            if self.error.get('caused_by'):
                self.cause_by_type = self.error.get('caused_by')['type']
                self.cause_by_reason = self.error.get('caused_by')['reason']
                self.cause_by_type_nested = self.error.get('caused_by')['caused_by']['type']
                self.cause_by_reason_nested = self.error.get('caused_by')['caused_by']['reason']
            self.status = error.get('status')

    def __eq__(self, other):
        if not isinstance(other, SnapshotError):
            self._log.error(NotImplemented)
            return NotImplemented
        return self.error == other.error



import json
import unittest

from automon.integrations.neo4j import Neo4jWrapper


class Neo4jTest(unittest.TestCase):

    def test_Neo4jWrapper(self):
        self.assertTrue(Neo4jWrapper)
        self.assertTrue(Neo4jWrapper())

# if __name__ == '__main__':
#     unittest.main()



import unittest

from automon.helpers.assertions import *
from automon.helpers.sleeper import *
from automon.helpers.sanitation import *


class SanitationTest(unittest.TestCase):
    def test_Sanitation(self):
        self.assertTrue(Sanitation)
        self.assertTrue(Sanitation.strip_quotes)
        self.assertTrue(Sanitation.strip_spaces)
        self.assertTrue(Sanitation.strip_spaces_from_list)
        self.assertTrue(Sanitation.safe_string)
        self.assertTrue(Sanitation.dedup)
        self.assertTrue(Sanitation.list_from_string)

        self.assertRaises(TypeError, Sanitation.strip_quotes)
        self.assertRaises(TypeError, Sanitation.strip_spaces)
        self.assertRaises(TypeError, Sanitation.strip_spaces_from_list)
        self.assertRaises(TypeError, Sanitation.safe_string)
        self.assertRaises(TypeError, Sanitation.dedup)
        self.assertRaises(TypeError, Sanitation.list_from_string)

        self.assertEqual(Sanitation.strip_quotes('test'), 'test')
        self.assertEqual(Sanitation.strip_spaces(' t e s t '), 't e s t')
        self.assertEqual(Sanitation.strip_spaces(' t e s t '), 't e s t')
        self.assertEqual(Sanitation.strip_spaces_from_list(
            [' t e s t ']), ['t e s t'])
        self.assertEqual(Sanitation.safe_string(
            'a;skdfjAS*&D)(&!H!:@JEN'), 'a_skdfjAS__D____H___JEN')
        self.assertEqual(Sanitation.dedup(
            [1, 1, 'AAAAA', 'AAAAA', 'AA', 555, 555, 5555, 6666, 7, 8, 33, 1, 1232, 499124, 'a']),
            [1, 'AAAAA', 'AA', 555, 5555, 6666, 7, 8, 33, 1232, 499124, 'a'])

        self.assertEqual(Sanitation.list_from_string(
            'a,d,a,3,1,5,g,2,,a,4,1,,h,4,1,k,z'),
            ['a', 'd', 'a', '3', '1', '5', 'g', '2', '', 'a', '4', '1', '', 'h', '4', '1', 'k', 'z']
        )
        self.assertEqual(Sanitation.list_from_string(
            'a d a 3 1 5 g 2  a 4 1  h 4 1 k z'),
            ['a', 'd', 'a', '3', '1', '5', 'g', '2', '', 'a', '4', '1', '', 'h', '4', '1', 'k', 'z']
        )
        self.assertEqual(Sanitation.list_from_string(
            ' ada315g2a41h41kz '),
            ['ada315g2a41h41kz']
        )


class SleeperTest(unittest.TestCase):
    def test_Sleeper(self):
        self.assertTrue(Sleeper)
        self.assertTrue(Sleeper.seconds)
        self.assertTrue(Sleeper.minute)
        self.assertTrue(Sleeper.within_a_minute)
        self.assertTrue(Sleeper.minutes)
        self.assertTrue(Sleeper.hour)
        self.assertTrue(Sleeper.hours)
        self.assertTrue(Sleeper.day)
        self.assertTrue(Sleeper.daily)
        self.assertTrue(Sleeper.time_range)

        self.assertRaises(TypeError, Sleeper.seconds)
        self.assertRaises(TypeError, Sleeper.minute)
        self.assertRaises(TypeError, Sleeper.within_a_minute)
        self.assertRaises(TypeError, Sleeper.minutes)
        self.assertRaises(TypeError, Sleeper.hour)
        self.assertRaises(TypeError, Sleeper.hours)
        self.assertRaises(TypeError, Sleeper.day)
        self.assertRaises(TypeError, Sleeper.daily)
        self.assertRaises(TypeError, Sleeper.time_range)
        self.assertRaises(TypeError, Sleeper.seconds, '1')

        self.assertIsNone(Sleeper.seconds(SleeperTest, 0))
        self.assertIsNone(Sleeper.minute(SleeperTest, 0))
        self.assertIsNone(Sleeper.within_a_minute(SleeperTest, 0))
        self.assertIsNone(Sleeper.minutes(SleeperTest, 0))
        self.assertIsNone(Sleeper.hour(SleeperTest, 0))
        self.assertIsNone(Sleeper.hours(SleeperTest, 0))
        self.assertIsNone(Sleeper.day(SleeperTest, 0))
        self.assertIsNone(Sleeper.daily(SleeperTest, 0))
        self.assertIsNone(Sleeper.time_range(SleeperTest, 0))


class AssertionsTest(unittest.TestCase):
    def test_make_tuple(self):
        self.assertTupleEqual(make_tuple('"test",'), ('test',))
        self.assertTupleEqual(make_tuple('"test","test"'), ('test', 'test'))
        self.assertRaises(ValueError, make_tuple, 'test')
        self.assertRaises(ValueError, make_tuple, 'test,')
        self.assertRaises(ValueError, make_tuple, 'test,test')

    def test_assert_label(self):
        self.assertEqual(assert_label(':test'), ':`test`')
        self.assertIsNone(assert_label('1test'))
        self.assertEqual(assert_label('works'), ':`works`')

    def test_assert_tuple(self):
        self.assertFalse(assert_tuple('test'))
        self.assertFalse(assert_tuple(1))
        self.assertFalse(assert_tuple(None))
        self.assertFalse(assert_tuple(object))
        self.assertFalse(assert_tuple(''))
        self.assertFalse(assert_tuple({}))
        self.assertFalse(assert_tuple(('test')))
        self.assertFalse(assert_tuple({1: 2}))
        self.assertTrue(assert_tuple(()))
        self.assertTrue(assert_tuple(('test',)))

    def test_assert_dict(self):
        self.assertFalse(assert_dict('test'))
        self.assertFalse(assert_dict({'test'}))
        self.assertTrue(assert_dict({'test': 'test'}))

    def test_assert_list(self):
        self.assertFalse(assert_list('test'))
        self.assertTrue(assert_list(['test']))
        self.assertTrue(assert_list([]))

    def test_assert_string(self):
        self.assertFalse(assert_string([]))
        self.assertFalse(assert_string(1))
        self.assertFalse(assert_string({}))
        self.assertFalse(assert_string(dict()))
        self.assertFalse(assert_string(tuple()))
        self.assertRaises(TypeError, assert_string)
        self.assertTrue(assert_string('test'))

# if __name__ == '__main__':
#     unittest.main()



import re
from ast import literal_eval

from automon.log.logger import Logging

log = Logging(__name__, Logging.DEBUG)


def make_tuple(obj: str) -> tuple:
    """Return a tuple from a string"""

    return literal_eval(obj)


def assert_label(label: str) -> str:
    """Make sure neo4j label is formatted correctly"""

    label = str(label)
    if label:
        if re.search('[:]', label):
            log.error(f"Invalid label '{label}': Remove the colon from the label")
            label = label.replace(':', '')

        if not re.search('[a-zA-Z]', label[0]):  # First letter of a label must be a letter
            log.error(f"Invalid label '{label}': First character of Neo4j :LABEL must be a letter")
        else:
            return f':`{label}`'  # :`Label`


def assert_tuple(obj: tuple or not tuple) -> bool:
    """Make sure it is a tuple"""

    if isinstance(obj, tuple):
        return True
    return False


def assert_dict(obj):
    """Make sure it is a dict"""

    try:
        if getattr(obj, 'fromkeys') \
                and getattr(obj, 'popitem') \
                and getattr(obj, 'setdefault') \
                and getattr(obj, 'update') \
                and getattr(obj, 'values'):
            return True
    except:
        return False


def assert_list(obj):
    """Make sure it is a list"""

    try:
        # isinstance(obj, list) and not isinstance(obj, str)
        if getattr(obj, 'append') and getattr(obj, 'sort') and getattr(obj, 'pop'):
            return True
    except:
        return False


def assert_string(obj):
    """Make sure it is a string"""

    try:
        # isinstance(obj, str)
        if getattr(obj, 'strip') and getattr(obj, 'split') and getattr(obj, 'rstrip'):
            return True
    except:
        return False



import unittest

from automon.log.logger import Logging, LogStream


class LoggerTest(unittest.TestCase):

    def test_logger(self):
        self.assertTrue(Logging())
        self.assertIsNone(Logging().error('test'))
        self.assertIsNone(Logging().debug('test'))
        self.assertIsNone(Logging().info('test'))
        self.assertIsNone(Logging().critical('test'))
        self.assertIsNone(Logging().warning('test'))

    def test_logstream(self):
        self.assertTrue(LogStream())
        self.assertFalse(f'{LogStream()}')
        self.assertIsNone(LogStream().flush())
        self.assertIsNone(LogStream().write('test'))

# if __name__ == '__main__':
#     unittest.main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="automonisaur", # Replace with your own username
    version="0.0.3",
    author="naisanza",
    author_email="naisanza@gmail.com",
    description="Core libraries for automonisaur",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TheShellLand/automon-core",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'requests==2.22.0',
        'Flask==1.1.1',
        'Flask-Login==0.4.1',
        'Flask-WTF==0.14.2',
        'bcrypt==3.1.7',
        'elasticsearch==7.10.0',
        'neo4j-driver==1.7.4',
        'slackclient==2.2.1',
        'python-swiftclient==3.9.0',
        'python-keystoneclient==3.22.0',
        'splunk-sdk==1.6.13',
        'pytz==2020.1',
        'nest-asyncio==1.3.3',
        'xmltodict==0.12.0',
        'pytest==5.4.3',
        'coverage==5.2',
        'pytest-cov==2.10.0',
    ]
)



import logging
import traceback

from logging import DEBUG, INFO, WARN, ERROR, CRITICAL, NOTSET

from automon.integrations.slack.slack_formatting import Chat, Format

# logging.basicConfig(format=log_format)

log = logging.getLogger('logger')
log.setLevel(CRITICAL)


class LogStream(object):
    """Allows logging to string
    """

    def __init__(self):
        self.logs = ''

    def write(self, string):
        self.logs += string

    def flush(self):
        pass

    def __str__(self):
        return self.logs


class Logging:
    """Standard logging to stdout
    """

    DEBUG = DEBUG
    INFO = INFO
    WARN = WARN
    ERROR = ERROR
    CRITICAL = CRITICAL
    NOTSET = NOTSET

    def __init__(self, name: __name__ = __name__, level: logging.INFO = INFO,
                 log_stream: LogStream = False):

        self.logging = logging.getLogger(name)
        self.logging.setLevel(level)

        # self.log_format = '%(levelname)s\t%(name)s\t%(module)s\t%(message)s'

        spacing = 4

        levelname = '%(levelname)s'
        modname = '%(name)s'
        message = '%(message)s'
        self.log_format = f'{levelname}\t{modname}\t{message}'
        # self.log_format = '%(levelname)s\t%(message)s\t%(name)s'

        logging.basicConfig(level=level, format=self.log_format)

        # TODO: need informative logging format
        # TODO: log streaming does not work
        if log_stream:
            self.stream = LogStream() if log_stream else None
            logging.basicConfig(level=level, stream=self.stream)

        log.debug('loaded')

    def error(self, msg: str = None, enable_traceback: bool = True):
        tb = traceback.format_exc()
        # tb = Chat.wrap(tb, Format.codeblock)
        if 'NoneType' not in tb and enable_traceback:
            self.logging.error(tb)

        if msg:
            return self.logging.error(msg)

    def warning(self, msg):
        return self.logging.warning(msg)

    def info(self, msg):
        return self.logging.info(msg)

    def debug(self, msg):
        return self.logging.debug(msg)

    def critical(self, msg):
        tb = traceback.format_exc()
        tb = Chat.wrap(tb, Format.codeblock)
        if 'NoneType' not in tb:
            self.logging.critical(tb)
        return self.logging.critical(msg)
